use crate::script_implementations::wasm::export_registry::register;
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::SpeciesLibrary;
use crate::static_data::{Form, Species};
use crate::StringKey;
use wasmer::FunctionEnvMut;

register! {

fn species_library_get_species(
    env: FunctionEnvMut<WebAssemblyEnv>,
    lib: ExternRef<dyn SpeciesLibrary>,
    string_key: ExternRef<StringKey>,
) -> ExternRef<dyn Species> {
    let lib = lib.value_func_box(&env).unwrap();
    let m = lib.get(string_key.value_func(&env).unwrap());
    if let Some(v) = m {
        ExternRef::func_new(&env, &v)
    } else {
        ExternRef::null()
    }
}

fn species_get_capture_rate(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> u8 {
    species.value_func_arc(&env).unwrap().capture_rate()
}

fn species_get_growth_rate(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> ExternRef<StringKey> {
    let species = species.value_func_arc(&env).unwrap();
    ExternRef::func_new(&env, species.growth_rate())
}

fn species_get_gender_rate(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> f32 {
    species.value_func_arc(&env).unwrap().gender_rate()
}

fn species_get_name(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> ExternRef<StringKey> {
    let species = species.value_func_arc(&env).unwrap();
    ExternRef::func_new(&env, species.name())
}

fn species_get_id(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> u16 {
    species.value_func_arc(&env).unwrap().id()
}

fn species_get_form_by_hash(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
    form_hash: u32
) -> ExternRef<dyn Form> {
        let species = species.value_func_arc(&env).unwrap();
        let form = species.get_form_by_hash(form_hash);
        if let Some(form) = form {
            ExternRef::func_new(&env, &form)
        } else {
            ExternRef::null()
        }
}

fn species_has_flag_by_hash(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
    flag_hash: u32
) -> u8 {
        if species.value_func_arc(&env).unwrap().has_flag_by_hash(flag_hash) { 1 } else { 0 }
}


}
